package com.oracle.coherence.swingdemo.agents;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

import com.oracle.coherence.util.tuples.TupleN;
import com.tangosol.io.ExternalizableLite;
import com.tangosol.net.AbstractInvocable;
import com.tangosol.net.CacheFactory;
import com.tangosol.net.CacheService;
import com.tangosol.net.Cluster;
import com.tangosol.net.DefaultConfigurableCacheFactory;
import com.tangosol.net.Invocable;
import com.tangosol.net.Member;
import com.tangosol.net.NamedCache;
import com.tangosol.util.ExternalizableHelper;

/**
 * An {@link Invocable} agent to determine the data distribution and status of each {@link Member} in the {@link Cluster} 
 * for a specific {@link NamedCache}
 */
public class DataDistributionQueryInvocable extends AbstractInvocable implements ExternalizableLite {

 	private static final long serialVersionUID = -3454331499261776141L;
 	
	private String cacheName;

    public DataDistributionQueryInvocable() { 
        //for ExternalizableLite
    }


    public DataDistributionQueryInvocable(String cacheName) {
        this.cacheName = cacheName;
    }


    /**
     * PRECONDITION: This invocable should only be executed against storage enable members
     **/
    public void run() {
        //calculate number of objects for the specified named cache
        int nrObjects = -1;
        NamedCache namedCache = CacheFactory.getCache(cacheName);
        if (namedCache != null) {
            CacheService cacheService = namedCache.getCacheService();
            if (cacheService.getBackingMapManager() instanceof DefaultConfigurableCacheFactory.Manager) {
                DefaultConfigurableCacheFactory.Manager backingMapManager = (DefaultConfigurableCacheFactory.Manager)cacheService.getBackingMapManager();
                nrObjects = backingMapManager.getBackingMap(cacheName).size();
            }
        }
        
        //we use the runtime to determine the current jvm memory state
        Runtime runtime = Runtime.getRuntime();
        
        //we need the member to send back membership information
        Member member = CacheFactory.getCluster().getLocalMember();
        
        setResult(new TupleN(nrObjects, member.getId(), runtime.maxMemory(), runtime.totalMemory(), runtime.freeMemory()));
    }


    public void readExternal(DataInput dataInput) throws IOException {
        cacheName = ExternalizableHelper.readSafeUTF(dataInput);
    }


    public void writeExternal(DataOutput dataOutput) throws IOException {
        ExternalizableHelper.writeSafeUTF(dataOutput, cacheName);
    }

}
