package com.oracle.coherence.util.tuples;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.Map;

import com.tangosol.util.ExternalizableHelper;

public class Pair<X, Y> implements Tuple {

	private static final long serialVersionUID = 2463266056607221797L;

	private X x;
	private Y y;
	
	public Pair() {
		//for ExternalizableLite
	}
	
	
	public Pair(X x, Y y) {
		this.x = x;
		this.y = y;
	}
	
	
	public Pair(Map.Entry<X, Y> entry) {
		this.x = entry.getKey();
		this.y = entry.getValue();
	}
	
	
	public Object get(int index) throws IndexOutOfBoundsException {
		if (index == 0)
			return x;
		else if (index == 1)
			return y;
		else
			throw new IndexOutOfBoundsException(String.format("%d is an illegal index for a Pair", index));
	}

	
	public int size() {
		return 2;
	}

	
	public X getX() {
		return x;
	}
	
	
	public Y getY() {
		return y;
	}
	
	
	@SuppressWarnings("unchecked")
	public void readExternal(DataInput dataInput) throws IOException {
		x = (X)ExternalizableHelper.readObject(dataInput);
		y = (Y)ExternalizableHelper.readObject(dataInput);
	}

	
	public void writeExternal(DataOutput dataOutput) throws IOException {
		ExternalizableHelper.writeObject(dataOutput, x);
		ExternalizableHelper.writeObject(dataOutput, y);
	}

	
	@Override
	public String toString() {
		return String.format("Pair(%s, %y)", x == null ? "null" : x.toString(), y == null ? "null" : y.toString());
	}
}
